"""App config for django_imdb."""

from django.apps import AppConfig


class DjangoImdbConfig(AppConfig):
    """App config for django_imdb."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "django_imdb"
